package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.TEXT_CONTINUE_NUMBERING;
import static w2phtml.office.XMLString.TEXT_LIST_HEADER;
import static w2phtml.office.XMLString.TEXT_LIST_ITEM;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import pro.litvinovg.xml.Debug;
import w2phtml.office.OfficeReader;

public class ListSplitter extends BasicSplitter implements ISplitter {

	public ListSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
	}

	@Override
	public boolean Split(Node list) {
		Node parent = list.getParentNode();
		Node listFirstPart = list.cloneNode(false);
		NodeList listNodes = list.getChildNodes();
		Document document = list.getOwnerDocument();
		Element softPageBreak = document.createElement(TEXT_SOFT_PAGE_BREAK);
		Node hangingItem = null;
		int i = 0;
		boolean dataMoved = false;
		while (listNodes.getLength() > i) {
			Node child = listNodes.item(i);
			if (child.getNodeType() == Node.ELEMENT_NODE) {
				String nodeName = child.getNodeName();
				if (nodeName.equals(TEXT_LIST_HEADER)) {
					if (containsSPB(child)) {
						// Remove inner SPB
						removeSPB(child);
						// HACK :(
						break;
					}
					listFirstPart.appendChild(child.cloneNode(true));
					// Get next element
					i++;
				} else if (nodeName.equals(TEXT_LIST_ITEM)) {
					if (containsSPB(child)) {

						// remove SPB, move previous nodes to firstPart.
						if (factory.split(child)) {
							dataMoved = true;
							// Add first part of list item to previous list item
							listFirstPart.appendChild(child.getPreviousSibling());
							if (child.hasChildNodes()) {
								hangingItem = child;
							} else {
								list.removeChild(child);
							}
						} 
						// Debug.printNode(child);

						break;
					} else {
						// Not with SPB yet, move node, set dataMoved=true
						listFirstPart.appendChild(child);
						dataMoved = true;
					}
				} else {
					System.out.println("Error. SPB in List child node " + nodeName);
					System.exit(1);
				}
			}
		}
		if (dataMoved) {
			((Element) list).setAttribute(TEXT_CONTINUE_NUMBERING, "true");
			parent.insertBefore(listFirstPart, list);
			parent.insertBefore(softPageBreak, list);
			if (hangingItem != null) {
				i = 0;
				NodeList freeNodes = hangingItem.getChildNodes();
				while (freeNodes.getLength() > 0) {
					Node freeNode = freeNodes.item(i);
					parent.insertBefore(freeNode, list);
				}
				list.removeChild(hangingItem);
			} 

		} else {
			//If SPB right before list items
			parent.insertBefore(softPageBreak, list);
		}

		return dataMoved;
	}

}
