package w2phtml.pageSplitters;

import static w2phtml.office.XMLString.FO_BREAK_BEFORE;
import static w2phtml.office.XMLString.TABLE_STYLE_NAME;
import static w2phtml.office.XMLString.TABLE_TABLE;
import static w2phtml.office.XMLString.TEXT_ALPHABETICAL_INDEX;
import static w2phtml.office.XMLString.TEXT_H;
import static w2phtml.office.XMLString.TEXT_ILLUSTRATION_INDEX;
import static w2phtml.office.XMLString.TEXT_LIST;
import static w2phtml.office.XMLString.TEXT_P;
import static w2phtml.office.XMLString.TEXT_SECTION;
import static w2phtml.office.XMLString.TEXT_SOFT_PAGE_BREAK;
import static w2phtml.office.XMLString.TEXT_STYLE_NAME;
import static w2phtml.office.XMLString.TEXT_TABLE_OF_CONTENT;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import pro.litvinovg.xml.Debug;
import w2phtml.office.OfficeReader;
import w2phtml.office.StyleWithProperties;
import w2phtml.util.Misc;

public class TextSplitter extends BasicSplitter implements ISplitter {

	public TextSplitter(OfficeReader officeReader, SplitFactory factory) {
		super(officeReader, factory);
	}

	@Override
	public boolean Split(Node onode) {
	//Find par node with soft page break inside and split it
			Document document = onode.getOwnerDocument();
			Element softPageBreak = document.createElement(TEXT_SOFT_PAGE_BREAK);
			NodeList nodes = onode.getChildNodes();
			
			int i = 0;
			//Loop through the content nodes and split nodes with soft page break
			while (i < nodes.getLength()){
				Node child = nodes.item(i);

				
				//Necessary check if node is an Element
				if (containsSPB(child)){
					String nodeName = child.getNodeName();
					//Create Duplicate Node!
					Element childFirstPart = (Element) child.cloneNode(false);
					StyleWithProperties style = null;
					if ((nodeName.equals(TEXT_P) || nodeName.equals(TEXT_H))) {
						//If SPB not the first node
						
						if (factory.split(child)){
							style = officeReader.getParStyle(Misc.getAttribute(child, TEXT_STYLE_NAME));
						}
					} else if (nodeName.equals(TABLE_TABLE)) {
						if (factory.split(child)){
							style = officeReader.getTableStyle(Misc.getAttribute(child, TABLE_STYLE_NAME));
						}
					} else if (nodeName.equals(TEXT_LIST)) {
						factory.split(child);
					} else if (nodeName.equals(TEXT_ILLUSTRATION_INDEX)) {
						factory.split(child);
					} else if (nodeName.equals(TEXT_ALPHABETICAL_INDEX)) {
						factory.split(child);
					} else if (nodeName.equals(TEXT_SECTION)) {
							if (factory.split(child)) {
								style = officeReader.getSectionStyle(Misc.getAttribute(child, TEXT_SECTION));
							}
					} else if (nodeName.equals(TEXT_TABLE_OF_CONTENT)){
						//HACK
						containsSPB(childFirstPart);
						i++;
						continue;
					} else if (nodeName.equals(TEXT_SOFT_PAGE_BREAK)){
						//HACK
						i++;
						continue;
					} else {
						Debug.printNode(child);
						System.exit(1);
					}
				
					if (style == null || !"page".equals(style.getProperty(FO_BREAK_BEFORE))){
						if (!nodeName.equals(TEXT_LIST)) {
							onode.insertBefore(softPageBreak.cloneNode(false), child);
						}
					}
					if (!child.hasChildNodes()){
						onode.removeChild(child);
					}
					continue;
				}
				i++;
			}
			//Debug.printNode(onode);
			return false;
	}

}
